<?php
/*
 This program is distributed under the terms of the GNU General Public License.
 See the README & COPYING files for more information.
 Copyright (C) 2004-2005  Kevin L. Papendick <kevinp@polarlava.com>
*/

//------------------------------------------------------------------
//--- CLASS: GzContent
//--- PURPOSE: GZip compress page contents for HTTP delivery
//---
//--- USAGE:
//---
/*
			<?php
				ob_start();
				require('GzContent.php');
			?>

			[ --- HTML Page Content --- ]

			<?php
				new GzContent();
			?>

*/
//--- ACKNOWLEDGEMENTS:
//--- 	The following sources were referenced in developing the
//--- 	HTTP output headers:
//---   		http://leknor.com/code/php/class.gzip_encode.php.txt
//---   		http://ppewww.ph.gla.ac.uk/~flavell/www/content-type.html
//------------------------------------------------------------------
class GzContent {

	//--- Class Variables
	var $debug = FALSE;					//debugging on/off
	var $debug_msg = array();			//debugging messages
	var $compression_level = 6;		//default output compression level
	var $enable = FALSE;					//can connection accept compressed output
	var $encoding;							//x-gzip or gzip encoding accepted


	//--------------------------------------------------
	//--- CONSTRUCTOR
	//--- PURPOSE:
	//---   Create a GzContent object
	//--- PARAMETERS:
	//---   level	-> OPTIONAL output compression level
	//---   debug	-> OPTIONAL enable/disable debug output
	//--- RETURNS: nothing
	//--------------------------------------------------
	function GzContent($level = '', $debug = '')
	{

		//--- Turn off implicit flushing
		ob_implicit_flush(0);

		//--- Set Compression Level
		if ($level != '') {
			$this->setCompressionLevel($level);
		}

		//--- Enable/Disable Debugging
		if ($debug != '') {
			$this->setDebug($debug);
		}

		//--- Check ability to compress output
		$this->setEnable($this->acceptGzOutput());

		//--- Attempt Content Compression
		$this->compressContent();
	}



	//-----------------------------------------------
	//--- METHOD: acceptGzOutput
	//--- PURPOSE: Determine if content can be compressed
	//---          and sets compression encoding.
	//--- PARAMETERS: none
	//--- RETURNS: TRUE/FALSE if content can be compressed
	//-----------------------------------------------
	function acceptGzOutput()
	{

		//--- Has output started or stopped?
		$stat = (headers_sent() || connection_aborted()) ? FALSE : TRUE;

		//--- Compression capability (x-gzip/gzip/none)
		if (isset($_SERVER['HTTP_ACCEPT_ENCODING']) && strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'x-gzip') !== FALSE) {
			$this->setEncoding('x-gzip');
			$stat = TRUE;
		} elseif (isset($_SERVER['HTTP_ACCEPT_ENCODING']) && strpos($_SERVER['HTTP_ACCEPT_ENCODING'],'gzip') !== FALSE) {
			$this->setEncoding('gzip');
			$stat = TRUE;
		} else {
			$this->setEncoding('none');
			$stat = FALSE;
		}

		return $stat;
	}


	//-----------------------------------------------
	//--- METHOD: compressContent
	//--- PURPOSE: Compress HTTP output
	//--- PARAMETERS: none
	//--- RETURNS: nothing
	//-----------------------------------------------
	function compressContent()
	{

		if ($this->getEnable()) {

			//--- Capture Output
			$output = '';
			$contents = ob_get_contents();

			//--- CRC & Size
			$size = strlen($contents);
			$crc = crc32($contents);

			//--- Build Output
			$this->setDebugMsg('Encoding Method - ' . $this->getEncoding() . '.');
			$output .= "\x1f\x8b\x08\x00\x00\x00\x00\x00";

			$this->setDebugMsg('Uncompressed Length ' . $size . ' bytes');

			//--- Compress
			$output .= gzcompress($contents, $this->getCompressionLevel());
			$this->setDebugMsg('Compressed Length - ' . strlen($output) . ' bytes.');

			$size = strlen($output);
			$output = substr($output, 0, strlen($output) - 4);
			$output .= pack('V', $crc) . pack('V', $size);

			ob_end_clean();

			//--- Send HTTP Headers
			header('Content-Encoding: ' . $this->getEncoding());
			header('Vary: Accept-Encoding'); //Required to prevent potential cache problems!
			header('Content-Length: ' . strlen($output));
			header('X-Content-Encoded-By: GzContent');

			//--- Send Debug Headers
			if ($this->getDebug()) {
				$this->getDebugMsg();
			}

			print $output;

		} else {

			//--- No Compression Supported
			$this->setDebugMsg('This browser does not support compression.');

			//--- Output Debug Message
			if ($this->getDebug()) {
				print $this->getDebugMsg();
			}

			//--- Flush Output Buffer
			ob_end_flush();
		}
	}


	//-----------------------------------------------
	//--- METHOD: setDebug
	//--- PURPOSE: enable/disable debugging
	//--- PARAMETERS: TRUE/FALSE
	//--- RETURNS: nothing
	//-----------------------------------------------
	function setDebug($a)
	{
		$this->debug = $a;
	}

	//-----------------------------------------------
	//--- METHOD: setDebugMsg
	//--- PURPOSE: Create/Append debug message
	//--- PARAMETERS: debug message
	//--- RETURNS: nothing
	//-----------------------------------------------
	function setDebugMsg($a)
	{
		array_push($this->debug_msg, $a);
	}

	//-----------------------------------------------
	//--- METHOD: setCompressionLevel
	//--- PURPOSE: sets compression level
	//--- PARAMETERS: 0-9 (min->max)
	//--- RETURNS: nothing
	//-----------------------------------------------
	function setCompressionLevel($a)
	{
		$this->compression_level = $a;
		$this->setDebugMsg('Compressed Level - ' . $a . '.');
	}

	//-----------------------------------------------
	//--- METHOD: setEncoding
	//--- PURPOSE: sets compression encoding
	//--- PARAMETERS: Encoding type
	//--- RETURNS: nothing
	//-----------------------------------------------
	function setEncoding($a)
	{
		$this->encoding = $a;
	}

	//-----------------------------------------------
	//--- METHOD: setEnable
	//--- PURPOSE: flag if accepting compressed output
	//--- PARAMETERS: TRUE/FALSE
	//--- RETURNS: nothing
	//-----------------------------------------------
	function setEnable($a)
	{
		$this->enable = $a;
	}




	//-----------------------------------------------
	//--- METHOD: getDebug
	//--- PURPOSE: Check enable/disable debugging
	//--- PARAMETERS: none
	//--- RETURNS: TRUE/FALSE
	//-----------------------------------------------
	function getDebug()
	{
		return $this->debug;
	}

	//-----------------------------------------------
	//--- METHOD: getDebugMsg
	//--- PURPOSE: Build debug message
	//--- PARAMETERS: none
	//--- RETURNS: HTML comment string
	//-----------------------------------------------
	function getDebugMsg()
	{

		$x = 0;
		foreach ($this->debug_msg as $entry) {
			header('X-GzContent Debug[' . $x . ']: ' . $entry);
			$x++;
		}

	}


	//-----------------------------------------------
	//--- METHOD: getCompressionLevel
	//--- PURPOSE: gets compression level
	//--- PARAMETERS: none
	//--- RETURNS: compression level
	//-----------------------------------------------
	function getCompressionLevel()
	{
		return $this->compression_level;
	}

	//-----------------------------------------------
	//--- METHOD: getEncoding
	//--- PURPOSE: gets compression encoding
	//--- PARAMETERS: none
	//--- RETURNS: Encoding type
	//-----------------------------------------------
	function getEncoding()
	{
		return $this->encoding;
	}

	//-----------------------------------------------
	//--- METHOD: getEnable
	//--- PURPOSE: check if accepting compressed output
	//--- PARAMETERS: none
	//--- RETURNS: TRUE/FALSE
	//-----------------------------------------------
	function getEnable()
	{
		return $this->enable;
	}

}
?>