<?php
/*
 This program is distributed under the terms of the GNU General Public License.
 See the README & COPYING files for more information.
 Copyright (C) 2006  Kevin L. Papendick <kevinp@polarlava.com>
*/

	//--- Includes
	require_once(LIB_PATH . 'PbBase_class.php');
	require_once(LIB_PATH . 'Localization_class.php');
	require_once(LIB_PATH . 'PbRbl_class.php');

	/**
	 * Comment Base Class
	 *
	 * This class provides common comment features support for comments and trackbacks.
	 *
	 * @package PolarBlog
	 * @author Kevin L. Papendick <kevinp@polarlava.com>
	 * @copyright 2006 Kevin L. Papendick <kevinp@polarlava.com>
	 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
	 * @version V1.0
	 */
	class PbCommentBase extends PbBase
	{
		//--- Class Variables
		/**
		 * Entry ID
		 * @var int
		 */
		var $eid;

		/**
		 * Commenter's Email
		 * @var string
		 */
		var $email;

		/**
		 * Comment Parent ID
		 * @var int
		 */
		var $pid;

		/**
		 * Commenter's Name
		 * @var string
		 */
		var $name;

		/**
		 * An integer string flag indicating if a comment should be displayed.
		 * @var string
		 */
		var $display;

		/**
		 * Comment timestamp, unformatted.
		 * @var string
		 */
		var $timestamp;

		/**
		 * Comment timestamp, formatted.
		 * @var string
		 */
		var $timestamp_formatted;

		/**
		 * Flag indicating if a comment exists.
		 * @var boolean
		 */
		var $comment_exists;

		/**
		 * Commenter's IP Address
		 * @var string
		 */
		var $ip;

		/**
		 * The language localization object.
		 * @var object
		 */
		var $langObj;

		/**
		 * List of RBL services to validate against.
		 * @var array
		 */
		var $rbl_services;

		/**
		 * RBL blocked/not blocked status of an inserted comment.
		 * @var boolean
		 */
		var $rbl_blocked;

		/**
		 * RBL Object
		 * @var object
		 */
		var $rblObj;

		/**
		 * Flags a commenter's IP address as being whitelisted.
		 * @var boolean
		 */
		var $ip_whitelisted;

		/**
		 * Flag that the comment is a trackback
		 * @var boolean
		 */
		var $is_trackback;

		/**
		 * Comment URL
		 * @var string
		 */
		 var $url;



		/**
		 * Comment Base class for other classes that need to inherit commentish functionality.
		 *
		 * @param string $prefix Database table prefix.
		 * @param array $rbls Array of RBL services to utilize
		 */
		function PbCommentBase($prefix = 'wl_', $rbls = array())
		{
			$this->PbBase($prefix);
			$this->langObj = new Localization(I18N_LANG, './lang/');
			$this->setRblServices($rbls);
			$this->setRblBlocked(FALSE);
		}


	//--------------------//
	//--- SET METHODS ----//
	//--------------------//

		/**
		 * Set flag indicating if a comment exists.
		 * @param boolean $a
		 */
		function setCommentExists($a)
		{
			$this->comment_exists = $a;
		}

		/**
		 * Set flag indicating if a comment should be displayed.
		 * @param string $a An integer string.
		 */
		function setDisplay($a)
		{
			$this->display = $a;
		}

		/**
		 * Set the entry id
		 *
		 * @param int $a
		 */
		function setEid($a)
		{
			$this->eid = (int) $a;
		}

		/**
		 * Set the parent id
		 *
		 * @param int $a
		 */
		function setPid($a)
		{
			$this->pid = (int) $a;
		}

		/**
		 * Set the commenter's name
		 *
		 * @param string $a
		 */
		function setName($a)
		{
			$this->name = $a;
		}

		/**
		 * Set comment IP address
		 *
		 * @param string $a
		 */
		function setIp($a)
		{
			$this->ip = $a;
		}

		/**
		 * Set comment is trackback flag
		 *
		 * @param boolean $a
		 */
		function setIsTrackback($a)
		{
			$this->is_trackback = $a;
		}

		/**
		 * Sets the IP address as whitelisted (TRUE) or blacklisted (FALSE).
		 *
		 * @param string $wl Whitelist value
		 */
		function setIpWhitelisted($wl)
		{
			$this->ip_whitelisted = (strlen($wl) == 0 || $wl == '1') ? TRUE : FALSE;
		}

		/**
		 * Create RBL Object
		 *
		 * @param string $prefix The database prefix string
		 * @param array $rbls List of RBL services to untilize
		 * @param bool $url_keyword_check Check referer URL for keywords
		 * @param bool $url_keyword_block Block IP on referer URL keyword match
		 */
		function setRblObj($prefix = 'wl_', $rbls = array(), $url_keyword_check = FALSE, $url_keyword_block = FALSE)
		{
			$this->rblObj = new PbRbl($prefix, $rbls, $url_keyword_check, $url_keyword_block, $this);
		}

		/**
		 * Sets the RBL blocked/not blocked status of an inserted comment.
		 *
		 * @param boolean $a Mark as blocked/not blocked
		 */
		function setRblBlocked($a)
		{
			$this->rbl_blocked = $a;
		}

		/**
		 * Sets RBL services for IP Checking
		 *
		 * @param array $a
		 *
		 */
		function setRblServices($a)
		{
			$this->rbl_services = $a;
		}

		/**
		 * Set comment timestamp
		 *
		 * @param string $a timestamp
		 * @param boolean $formatted Flag indicating timestamp is formatted.
		 */
		function setTimestamp($a, $formatted = FALSE)
		{
			if ($formatted) {
				$this->timestamp_formatted = $a;
			} else {
				$this->timestamp = $a;
			}
		}

		/**
		 * Set comment URL
		 *
		 * @param string $a
		 * @param boolean $strip Strips the URL scheme.
		 */
		function setUrl($a, $strip = TRUE)
		{
			$this->url = $a;
		}





		//-----------------------------------------------
		//-----------------------------------------------
		//--- GET METHODS -------------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		/**
		 * Get entry id
		 *
		 * @return int
		 */
		function getEid()
		{
			return (int) $this->eid;
		}

		/**
		 * Get parent id
		 *
		 * @return int
		 */
		function getPid()
		{
			return (int) $this->pid;
		}

		/**
		 * Get the commenter's name
		 *
		 * @return string
		 */
		function getName()
		{
			return $this->name;
		}

		/**
		 * Get flag indicating if a comment should be displayed.
		 * @return string
		 */
		function getDisplay()
		{
			return $this->display;
		}

		/**
		 * Get comment IP address
		 * @return string
		 */
		function getIp()
		{
			return $this->ip;
		}

		/**
		 * Get comment is trackback flag
		 * @return boolean
		 */
		function getIsTrackback()
		{
			return $this->is_trackback;
		}

		/**
		 * Returns TRUE if a commenter's IP address is whitelisted.
		 *
		 * @return boolean
		 */
		function getIpWhitelisted()
		{
			return $this->ip_whitelisted;
		}

		/**
		 * Get flag indicating if a comment exists.
		 * @return boolean
		 */
		function getCommentExists()
		{
			return $this->comment_exists;
		}

		/**
		 * Indicates if inserted comment was blocked by the RBL or not
		 *
		 * @return boolean
		 */
		function getRblBlocked()
		{
			return $this->rbl_blocked;
		}

		/**
		 * Retrieve RBL services for IP Checking
		 *
		 * @return array
		 *
		 */
		function getRblServices()
		{
			return $this->rbl_services;
		}

		/**
		 * Get comment timestamp
		 *
		 * @param boolean $formatted Flag indicating if timestamp should be formatted.
		 * @return string
		 */
		function getTimestamp($formatted = FALSE)
		{
			if ($formatted) {
				$date = $this->timestamp_formatted;
			} else {
				$date = $this->timestamp;
			}
			return $date;
		}

		/**
		 * Get comment URL
		 *
		 * @return string
		 */
		function getUrl()
		{
			return $this->url;
		}




		//-----------------------------------------------
		//-----------------------------------------------
		//--- OTHER METHODS -----------------------------
		//-----------------------------------------------
		//-----------------------------------------------


		/**
		 * Inserts Comments and Trackbacks into the comment table.
		 *
		 * @param boolean $is_trackback Flag that comment is a trackback
		 * @return boolean
		 */
		function insertData($is_trackback = FALSE)
		{
			$this->setIsTrackback($is_trackback);
			$trackback = ($is_trackback) ? '1' : '0';
			$query = "INSERT INTO
							{$this->prefix}comments
							(
							 eid,
							 pid,
							 name,
							 email,
							 url,
							 display,
							 comment,
							 ip,
							 trackback
							)
						VALUES
							(
							 '%i',
							 '%i',
							 '%s',
							 '%s',
							 '%s',
							 '%s',
							 '%s',
							 '%s',
							 '%s'
							)";

			if ($is_trackback) {
				$query_data = array(
									$this->getEid(),
									$this->getPid(),
									$this->getTbBlogName(),
									$this->getTbTitle(),
									$this->getUrl(),
									$this->getDisplay(),
									$this->getTbExcerpt(),
									$this->getUserIp(),
									($this->getIsTrackback()) ? '1' : '0'
								  );
            } else {
				$query_data = array(
									$this->getEid(),
									$this->getPid(),
									$this->getName(),
									$this->getEmail(),
									$this->getUrl(),
									$this->getDisplay(),
									$this->getComment(),
									$this->getUserIp(),
									($this->getIsTrackback()) ? '1' : '0'
								  );
			}

			$stat = $this->dbObj->safeQuery($query, $query_data);
			if ($stat) {
				if (DB_CONNECT === 'mysqli') {
					$this->setId(mysqli_insert_id($this->dbObj->getDb()));
				} else {
					$this->setId(mysql_insert_id());
				}
				$this->setCommentExists($stat);
			}

			return $stat;
		}

		/**
		 * Checks if comments are allowed on an entry.
		 *
		 * @param int $eid Entry ID
		 * @return boolean
		 */
		function commentsEnabled($eid)
		{
			$stat = FALSE;
			$query = "SELECT
							comments
						 FROM
						 	{$this->prefix}entries
						 WHERE
						 	eid='%i'";
				$query_data = array($eid);

				$stat = $this->dbObj->safeQuery($query, $query_data);
				if ($stat) {
					$row = $this->dbObj->getRow();
					$stat = ((int) $row['comments']) ? TRUE : FALSE;
				}

			return $stat;
		}

		/**
		 * Notify entry or commentor of comment/trackback
		 *
		 * @param string $by_name Commentor's name
		 * @param boolean $comment Comment or reply flag.
		 */
		function emailNotify($by_name, $comment = TRUE)
		{
			if (!MAIL_TYPE) {
				return;
			}

			$mail_debug_msg = '';
			$blog_name = (defined('BLOG_NAME')) ? BLOG_NAME : 'PolarBlog';
			if ($comment && $this->getIsTrackback()) {
				$mail_subject = stripslashes(strip_tags($blog_name)) . ' ' . $this->langObj->get('TRACKBACK') . ': ';
				$mail_msg = $this->langObj->get('MAIL_TB_INTRO') . ' ';
				$query = "SELECT
								u.post_by AS name,
								u.email,
								e.title
							 FROM
								{$this->prefix}entries AS e,
								{$this->prefix}users AS u
							 WHERE
								e.eid='%i'
							 AND
								u.id=e.uid;";
				$query_data = array(
									$this->getEid()
								  );
			} elseif ($comment) {
				$mail_subject = stripslashes(strip_tags($blog_name)) . ' ' . $this->langObj->get('MAIL_SUBJECT_COMMENT') . ': ';
				$mail_msg = $this->langObj->get('MAIL_COMMENT_INTRO') . ' ';
				$query = "SELECT
								u.post_by AS name,
								u.email,
								e.title
							 FROM
								{$this->prefix}entries AS e,
								{$this->prefix}users AS u
							 WHERE
								e.eid='%i'
							 AND
								u.id=e.uid;";
				$query_data = array($this->getEid());
			} else {
				$mail_subject = stripslashes(strip_tags($blog_name)) . ' ' . $this->langObj->get('MAIL_SUBJECT_REPLY') . ': ';
				$mail_msg = $this->langObj->get('MAIL_REPLY_INTRO') . ' ';
				$query = "SELECT
							c.name,
							c.email,
							e.title
						  FROM
						  	{$this->prefix}comments AS c,
							{$this->prefix}entries AS e
						  WHERE
						  	cid='%i'
						  AND
							e.eid=c.eid";
				$query_data = array($this->getPid());
			}

			if ($this->dbObj->safeQuery($query, $query_data)) {
				$row = $this->dbObj->getRow();
				$mail_subject .= $row['title'];
				$mail_msg = str_replace('::NAME::', $by_name, $mail_msg);
				$mail_msg .= $row['title'] . ' ' . $this->langObj->get('MAIL_OUTRO') . ': ';
				$mail_msg .= BASE_URL . '?eid=' . $this->getEid() . '#' . $this->getId() . "\r\n\r\n";
				$disabled = ($this->getDisplay() == 0) ? ' (' . $this->langObj->get('DISPLAY_DISABLED') . ')' : '';
				$mail_msg .= '--- ' . $this->langObj->get('MAIL_THE_COMMENT') . "$disabled:\r\n";

				if ($this->getIsTrackback()) {
					$mail_msg .= $this->getTbExcerpt() . "\r\n\r\n\r\n";;
				} else {
					//--- Convert <br /> -> "\r\n"
					$the_comment = str_replace('<br />', "\r\n", $this->getComment('DO_STUPEFY'));
					$mail_msg .= stripslashes(strip_tags($the_comment)) . "\r\n\r\n\r\n";
				}

				$for = $blog_name;
				$for .= ' (' . BASE_URL . ')';
				$for = stripslashes(strip_tags($for));
				$mail_msg .= $this->langObj->get('MAIL_TAG') . " $for.\r\n";

				if (defined('COMMENT_MAIL') && COMMENT_MAIL) {
					require_once(LIB_PATH . 'class.phpmailer.php');
					$mail = new PHPMailer();
					if (defined('MAIL_HOST')) {
						$mail->Host = MAIL_HOST;
						$mail_debug_msg = 'Host - ' . $mail->ErrorInfo . '<br />';
					}
					$mail->AddCustomHeader('X-Tracker: PolarBlog Notifier');
					$mail->CharSet = I18N_CHARSET;
					$mail_debug_msg .= 'Charset - ' . $mail->ErrorInfo . '<br />';
					$mail->Mailer = MAIL_TYPE;
					$mail_debug_msg .= 'Mailer - ' . $mail->ErrorInfo . '<br />';
					$mail->From = MAIL_FROM;
					$mail_debug_msg .= 'From - ' . $mail->ErrorInfo . '<br />';
					$mail->FromName = MAIL_FROM_NAME;
					$mail_debug_msg .= 'FromName - ' . $mail->ErrorInfo . '<br />';
					$mail->Subject = $mail_subject;
					$mail_debug_msg .= 'Subject - ' . $mail->ErrorInfo . '<br />';

					$send_mail = FALSE;
					if (strlen($row['email']) > 0) {
						$mail->AddAddress($row['email'], $row['name']);
						$mail_debug_msg .= 'AddAddress - ' . $mail->ErrorInfo . '<br />';
						$mail->AddReplyTo($row['email'], $row['name']);
						$mail_debug_msg .= 'AddReplyTo - ' . $mail->ErrorInfo . '<br />';
						$send_mail = TRUE;
					}
					if (!$comment && defined('MAIL_BCC_REPLY')) {
						if (strlen($row['email']) > 0) {
							$mail->AddBCC(MAIL_FROM, MAIL_FROM_NAME);
							$mail_debug_msg .= 'AddBCC - ' . $mail->ErrorInfo . '<br />';
						} else {
							$mail->AddAddress(MAIL_FROM, MAIL_FROM_NAME);
							$mail_debug_msg .= 'AddAddress(2) - ' . $mail->ErrorInfo . '<br />';
						}
						$send_mail = TRUE;
					}
					$mail->Body = $mail_msg;
					$mail_debug_msg .= 'Body - ' . $mail->ErrorInfo . '<br />';

					if ($send_mail) {
						$stat = $mail->Send();
						$mail_debug_msg .= 'Send - ' . $mail->ErrorInfo . '<br />';
					}

					if (defined('MAIL_DEBUG') && MAIL_DEBUG) {
						echo 'Mail Debug:<br />' . $mail_debug_msg;
					}
				}
			}
		}

	} //end of class
?>
